/* *************************************** */
/*    File I/O Interface Class ver.2.00    */
/*                          'fileio.cpp'   */
/*                        by T.Nakayama    */
/* *************************************** */

#include "fileio.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

FileIO :: FileIO()
 : 	_dwError(FILEIO_COMPLETE), _bErrorMode(FILEIO_ERRORMODE_NATIVE)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

FileIO :: ~FileIO()
{
}

/* ************************************************************************* */
/*    Set Last Error                                                         */
/* ************************************************************************* */

DWORD FileIO :: SetLastError(DWORD dwError)
{
	DWORD	dwErrorOld;

	dwErrorOld = _dwError;
	_dwError = dwError;
	return(dwErrorOld);
}

/* ************************************************************************* */
/*    Get Last Error                                                         */
/* ************************************************************************* */

inline DWORD FileIO :: GetLastError(void)
{
	return(_dwError);
}

/* ************************************************************************* */
/*    Set Error Mode                                                         */
/* ************************************************************************* */

BOOL FileIO :: SetErrorMode(BOOL bErrorMode)
{
	BOOL	bErrorModeOld;

	bErrorModeOld = _bErrorMode;
	_bErrorMode = bErrorMode;
	return(bErrorModeOld);
}

/* ************************************************************************* */
/*    Get Error Mode                                                         */
/* ************************************************************************* */

inline BOOL FileIO :: GetErrorMode(void)
{
	return(_bErrorMode);
}

/* ************************************************************************* */
/*    Open File                                                              */
/* ************************************************************************* */

HANDLE	FileIO :: OpenFile(LPCTSTR pszFileName,
	DWORD dwDesireAccess, DWORD dwAttribute, DWORD dwCreate)
{
	HANDLE	hFile;

	hFile = CreateFile(pszFileName, dwDesireAccess, 0, NULL,
		dwCreate, dwAttribute, NULL);
	if(hFile == INVALID_HANDLE_VALUE){
		if(GetErrorMode())
			SetLastError(:: GetLastError());
		else
			SetLastError(FILEIO_CANNOT_OPEN_FILE);
		return(NULL);
	}
	return(hFile);
}

/* ************************************************************************* */
/*    Close Handle                                                           */
/* ************************************************************************* */

BOOL FileIO :: CloseHandle(HANDLE hFile)
{
	if(!:: CloseHandle(hFile)){
		if(GetErrorMode())
			SetLastError(:: GetLastError());
		else
			SetLastError(FILEIO_FAILED);
		return(FALSE);
	}
	return(TRUE);
}

/* ************************************************************************* */
/*    Get File Size                                                          */
/* ************************************************************************* */

DWORD FileIO :: GetFileSize(HANDLE hFile, LPDWORD pdwFileSizeHigh)
{
	DWORD	dwSize;

	dwSize = :: GetFileSize(hFile, pdwFileSizeHigh);
	if(dwSize == 0xffffffff)
		if(GetErrorMode())
			SetLastError(:: GetLastError());
		else
			SetLastError(FILEIO_FAILED);
	return(dwSize);
}

/* ************************************************************************* */
/*    Read File                                                              */
/* ************************************************************************* */

DWORD FileIO :: ReadFile(HANDLE hFile, LPVOID pBuffer, DWORD dwSize)
{
	DWORD	dwRead;

	if(!:: ReadFile(hFile, pBuffer, dwSize, &dwRead, NULL))
		if(GetErrorMode())
			SetLastError(:: GetLastError());
		else
			SetLastError(FILEIO_CANNOT_READ_FILE);
	return(dwRead);
}

/* ************************************************************************* */
/*    Write File                                                             */
/* ************************************************************************* */

DWORD FileIO :: WriteFile(HANDLE hFile, LPVOID pBuffer, DWORD dwSize)
{
	DWORD	dwWrite;

	if(!:: WriteFile(hFile, pBuffer, dwSize, &dwWrite, NULL))
		if(GetErrorMode())
			SetLastError(:: GetLastError());
		else
			SetLastError(FILEIO_CANNOT_WRITE_FILE);
	return(dwWrite);
}

