/* ********************************************************* */
/*    Transform Wave File to PCM for FM-7 Source ver.1.00    */
/*                                         'wav2pcm7.cpp'    */
/* ********************************************************* */

#include "wav2pcm7.h"

/* ********** */
/*    Main    */
/* ********** */

int main(int argc,char *argv[])
{
	int					i, iFileLen, iSize;
	int					iPcmWait = -1, iRetCode = 0;
	DWORD				dwSampleRate, dwFileSize = 0;
	LPTSTR				pszWaveFile = NULL, pszBinFile = NULL;
	LPTSTR				pszWaveFileTmp = NULL, pszBinFileTmp = NULL;
	LPTSTR				pszDrive = NULL, pszDir = NULL, pszFname = NULL;
	LPTSTR				pszWaveExt = NULL, pszBinExt = NULL;
	LPBYTE				pFile = NULL, pPcm = NULL, pWave = NULL, pData = NULL;
	BOOL				flgExit = FALSE;
	TCHAR				c;

	// Show Version
	ShowVersion();

	// Check Number of Options
	if(argc <= 1){
		iRetCode = RETCODE_PARAMETER_NOTHING;
		goto ERROR_PROC;
	}

	// Decode Option
	for(i = 1; i < argc; i++){
		c = *((LPTSTR)(argv[i]));
		if(c == '/' || c == '-'){
			c = *((LPTSTR)(argv[i]) + 1);
			switch(c){
				case 'w':		// Set Wait
				case 'W':
					iPcmWait = atoi((LPCTSTR)(argv[i] + 2));
					if(iPcmWait < 0)
						iPcmWait = 0;
					if(iPcmWait > 255)
						iPcmWait = 255;
					break;

				case '?':		// Usage
				case 'h':
				case 'H':
					ShowUsage();
					flgExit = TRUE;
					break;
			}
		}
		else{
			if(!pszWaveFileTmp){	// Get Wave (input) File Name
				pszWaveFileTmp = new TCHAR[strlen(argv[i]) + 1];
				strcpy(pszWaveFileTmp, argv[i]);
			}
			else{
				if(!pszBinFileTmp){		// Get Binary (output) File Name
					pszBinFileTmp = new TCHAR[strlen(argv[i]) + 1];
					strcpy(pszBinFileTmp, argv[i]);
				}
			}
		}
	}
	if(flgExit){
		_DELETEARRAY(pszWaveFileTmp);
		_DELETEARRAY(pszBinFileTmp);
		return(iRetCode);
	}

	// Check Wave File Name
	if(!pszWaveFileTmp){
		iRetCode = RETCODE_WAVEFILENAME_NOTHING;
		goto ERROR_PROC;
	}

	// Analyze Wave File Name
	iFileLen = strlen(pszWaveFileTmp);
	pszDrive = new TCHAR[iFileLen + 1];
	pszDir = new TCHAR[iFileLen + 1];
	pszFname = new TCHAR[iFileLen + 1];
	pszWaveExt = new TCHAR[iFileLen + 1];
	_splitpath(pszWaveFileTmp, pszDrive, pszDir, pszFname, pszWaveExt);
	if(!strlen(pszWaveExt))
		strcpy(pszWaveExt, EXTENSION_WAVEFILE);
	iFileLen = strlen(pszDrive) + strlen(pszDir) + strlen(pszFname)
		+ strlen(pszWaveExt);
	pszWaveFile = new TCHAR[iFileLen + 1];
	_makepath(pszWaveFile, pszDrive, pszDir, pszFname, pszWaveExt);

	// Analyze Bin File Name;
	if(!pszBinFileTmp){
		// Create Bin File Name
		pszBinExt = new TCHAR[strlen(EXTENSION_BINFILE) + 1];
		strcpy(pszBinExt, EXTENSION_BINFILE);
	}
	else{
		// Check Extension
		_DELETEARRAY(pszDrive);
		_DELETEARRAY(pszDir);
		_DELETEARRAY(pszFname);
		iFileLen = strlen(pszBinFileTmp);
		pszDrive = new TCHAR[iFileLen + 1];
		pszDir = new TCHAR[iFileLen + 1];
		pszFname = new TCHAR[iFileLen + 1];
		pszBinExt = new TCHAR[iFileLen + 1];
		_splitpath(pszBinFileTmp, pszDrive, pszDir, pszFname, pszBinExt);
		if(!strlen(pszBinExt))
			// Add Extension
			strcpy(pszBinExt, EXTENSION_BINFILE);
	}

	// Make Bin File Name
	iFileLen = strlen(pszDrive) + strlen(pszDir) + strlen(pszFname)
		+ strlen(pszBinExt);
	pszBinFile = new TCHAR[iFileLen + 1];
	_makepath(pszBinFile, pszDrive, pszDir, pszFname, pszBinExt);

	// Delete Temporary Buffer
	_DELETEARRAY(pszDrive);
	_DELETEARRAY(pszDir);
	_DELETEARRAY(pszFname);
	_DELETEARRAY(pszWaveExt);
	_DELETEARRAY(pszBinExt);
	_DELETEARRAY(pszWaveFileTmp);
	_DELETEARRAY(pszBinFileTmp);

	// Read Wave File
	if(ReadFile(pszWaveFile, &pFile, &dwFileSize)){
		iRetCode = RETCODE_WAVEFILE_CANNOT_OPEN;
		goto ERROR_PROC;
	}

	// Check Wave Format
	if(iRetCode = CheckWaveFmt(pFile, dwFileSize)){
		switch(iRetCode){
			case 0x105:
				iRetCode = RETCODE_ILLEGAL_FORMAT_TAG;
				break;

			case 0x106:
				iRetCode = RETCODE_ILLEGAL_CHANNEL_NUMBER;
				break;

			case 0x107:
				iRetCode = RETCODE_ILLEGAL_BITS;
				break;

			default:
				iRetCode = RETCODE_NOT_WAVEFILE;
				break;
		}
		goto ERROR_PROC;
	}

	// Set PCM Wait
	// OptionŐݒ肳ĂꍇSkip
	if(iPcmWait < 0){
		//  Get PCM Wait
		dwSampleRate = GetSampleRate(pFile);
		iPcmWait = GetPcmWait(dwSampleRate);
	}

	// Get Wave Data Size
	iSize = (int)GetWaveSize(pFile);

	// Create PCM Data Buffer
	pData = new BYTE[iSize + 12];
	memset(pData, 0, iSize + 12);
	pWave = pFile + sizeof(SWAVEFILEHEADER);
	pPcm = pData + 6;

	// Set Machine Language Header
	*pData = 0x00;
	*(pData + 1) = (BYTE)(iSize >> 8);
	*(pData + 2) = (BYTE)(iSize & 0xff);
	*(pData + 3) = 0x00;
	*(pData + 4) = 0x00;
	*(pData + 5) = (BYTE)(iPcmWait & 0xff);	// wait
	*(pData + iSize + 6) = 0xff;
	*(pData + iSize + 7) = 0x00;
	*(pData + iSize + 8) = 0x00;
	*(pData + iSize + 9) = 0x00;
	*(pData + iSize + 10) = 0x00;
	*(pData + iSize + 11) = 0x1a;

	// Copy Wave Data to PCM
	for(i = 0; i < iSize; i++)
		*pPcm++ = *pWave++;

	// Write Bin File
	if(WriteFile(pszBinFile, pData, iSize + 11)){
		iRetCode = RETCODE_BINFILE_CANNOT_OPEN;
		goto ERROR_PROC;
	}

ERROR_PROC:

	// Show Log & Error Message
	if(!iRetCode)
		// Show Log
		fprintf(stdout, "Ok: %s -> %s\n", pszWaveFile, pszBinFile);
	else
		// Show Error Message
		ShowError(iRetCode, pszWaveFile, pszBinFile);

	// Delete Allocated Memory
	_DELETEARRAY(pData);
	_DELETEARRAY(pFile);
	_DELETEARRAY(pszWaveFile);
	_DELETEARRAY(pszBinFile);

	return(iRetCode);
}

/* *************** */
/*    Read File    */
/* *************** */

DWORD ReadFile(LPCTSTR pFileName, LPBYTE *pFile, DWORD *dwFileSize)
{
	LPFileIO			pFileIO;
	HANDLE				hFile;
	DWORD				dwError = 0;

	pFileIO = new FileIO;
	hFile = pFileIO->OpenFile(pFileName);
	if(hFile == NULL)
		dwError = pFileIO->GetLastError();

	else{
		*dwFileSize = pFileIO->GetFileSize(hFile);
		*pFile = new BYTE[*dwFileSize];
		pFileIO->ReadFile(hFile, *pFile, *dwFileSize);
		pFileIO->CloseHandle(hFile);
	}
	_DELETE(pFileIO);

	return(dwError);
}

/* **************** */
/*    Write File    */
/* **************** */

DWORD WriteFile(LPCTSTR pFileName, LPBYTE pFile, DWORD dwFileSize)
{
	LPFileIO			pFileIO;
	HANDLE				hFile;
	DWORD				dwError = 0;

	pFileIO = new FileIO;
	hFile = pFileIO->OpenFile(pFileName,
			GENERIC_WRITE, FILE_ATTRIBUTE_NORMAL, CREATE_ALWAYS);
	if(hFile == NULL)
		dwError = pFileIO->GetLastError();

	else{
		pFileIO->WriteFile(hFile, (LPVOID)pFile, dwFileSize);
		pFileIO->CloseHandle(hFile);
	}
	_DELETE(pFileIO);

	return(dwError);
}

/* ****************************** */
/*    Check Wave Header Format    */
/* ****************************** */

DWORD CheckWaveFmt(LPBYTE pData, DWORD dwSize)
{
	DWORD				dwError = 0;
	LPSWAVEFILEHEADER	pWaveInfo;
	LPSWAVEFORMAT		pWaveFmt;

	pWaveInfo = (LPSWAVEFILEHEADER)pData;

	// Check File Size
	if(dwSize != pWaveInfo->dwFileSize + 8){
		dwError = 0x101;
		return(dwError);
	}

	// Check Header String
	if(strncmp((LPCSTR)pWaveInfo->abyRiff, "RIFF", 4))
		dwError = 0x102;

	if(strncmp((LPCSTR)pWaveInfo->abyWave, "WAVE", 4))
		dwError = 0x103;

	if(strncmp((LPCSTR)pWaveInfo->abyFmt, "fmt ", 4))
		dwError = 0x104;

	pWaveFmt = &pWaveInfo->stWaveFormat;

	// Check Format Tag
	if(pWaveFmt->wFormatTag != 1)
		dwError = 0x105;

	// Check Number of Channel
	if(pWaveFmt->wChannels != 1)
		dwError = 0x106;

	// Check Bits Per Sample
	if(pWaveFmt->wBitsPerSample != 8)
		dwError = 0x107;

	return(dwError);
}

/* ********************* */
/*    Get Sample Rate    */
/* ********************* */

inline DWORD GetSampleRate(LPBYTE pData)
{
	return(((LPSWAVEFILEHEADER)pData)->stWaveFormat.dwSamplesPerSec);
}

/* ************************ */
/*    Get Wave Data Size    */
/* ************************ */

inline DWORD GetWaveSize(LPBYTE pData)
{
	return(((LPSWAVEFILEHEADER)pData)->dwWaveSize);
}

/* ****************** */
/*    Get PCM Wait    */
/* ****************** */

int GetPcmWait(DWORD dwSampleRate)
{
	int		iWait;
	DWORD	dwSample[] ={
				12493, 12073, 11681, 11314, 10969, 10644, 10339, 10050,
				9777 , 9518 , 9273 , 9040 , 8818 , 8607 , 8406 , 8214 ,
				8031 , 7855 , 7688 , 7527 , 7372 , 7224 , 7082 , 6945 ,
				6814 , 6687 , 6565 , 6448 , 6334 , 6224 , 6119 , 6016 ,
				5917 , 5822 , 5729 , 5639 , 5552 , 5468 , 5386 , 5306 ,
				5229 , 5154 , 5081 , 5011 , 4942 , 4875 , 4810 , 4746 ,
				4684 , 4624 , 4565 , 4508 , 4452 , 4398 , 4345 , 4293 ,
				4242 , 4193 , 4145 , 4097 , 4051 , 4006 , 3962 , 3919};

	for(iWait = 0; iWait < 64; iWait++){
		if(dwSampleRate > dwSample[iWait])
			break;
	}
	if(iWait > 0)
		iWait--;

	return(iWait);
}

/* ****************** */
/*    Show Version    */
/* ****************** */

int ShowVersion(void)
{
	fprintf(stderr, "Transform Wave File to PCM for FM-7\n");
	fprintf(stderr, "                       Version 1.00\n");
	fprintf(stderr, "           Programmed by T.Nakayama\n");
	fprintf(stderr, "\n");

	return(0);
}

/* **************** */
/*    Show Usage    */
/* **************** */

int ShowUsage(void)
{
	fprintf(stderr, "Usage:\n");
	fprintf(stderr, "wav2pcm7 wave-filename[.wav] [output-filename[.bin]]" \
		" [{-|/}option]\n");
	fprintf(stderr,"\n");
	fprintf(stderr,"Option\n");
	fprintf(stderr," -w:PCM Wait [0 - 255]\n");
	fprintf(stderr," -?:Help\n");
	fprintf(stderr,"\n");

	return(0);
}

/* **************** */
/*    Show Error    */
/* **************** */

int ShowError(int iError, LPCTSTR pszWaveFile, LPCTSTR pszBinFile)
{
	if(!iError)
		return(iError);

	fprintf(stderr, "err: ");
	switch(iError){
		case RETCODE_PARAMETER_NOTHING:
			fprintf(stderr, "p[^w肳Ă܂B\n");
			break;

		case RETCODE_WAVEFILENAME_NOTHING:
			fprintf(stderr, "͂WAVEt@Cw肳ĂB\n");
			break;

		case RETCODE_WAVEFILE_CANNOT_OPEN:
			fprintf(stderr, "̓t@C̃I[vɎs܂B(%s)\n",
				pszWaveFile);
			break;

		case RETCODE_NOT_WAVEFILE:
			fprintf(stderr, "WAVEt@Cł͂܂B(%s)\n", pszWaveFile);
			break;

		case RETCODE_ILLEGAL_FORMAT_TAG:
			fprintf(stderr, "WAVEt@C̃I[fBI`" \
				"jAPCMł͂܂B(%s)\n", pszWaveFile);
			break;

		case RETCODE_ILLEGAL_CHANNEL_NUMBER:
			fprintf(stderr, "WAVEt@C̃`l2ȏłB(%s)\n",
				pszWaveFile);
			break;

		case RETCODE_ILLEGAL_BITS:
			fprintf(stderr, "WAVEt@C1Tṽrbg" \
				"8bitł͂܂B(%s)\n", pszWaveFile);
			break;

		case RETCODE_BINFILE_CANNOT_OPEN:
			fprintf(stderr, "o̓t@C̃I[vɎs܂B(%s)\n",
				pszBinFile);
			break;

		default:
			fprintf(stderr, "sȃG[܂B" \
				"(InputFile:%s, OutputFile:%s)\n", pszWaveFile, pszBinFile);
			break;
	}
	fprintf(stderr, "\n");

	return(iError);
}

